package main

import (
	"fmt"
	"net/http"

	"github.com/gin-gonic/gin"
	swaggerFiles "github.com/swaggo/files"
	ginSwagger "github.com/swaggo/gin-swagger"
	_ "github.com/timetracker/backend/docs" // This line is important for swag to work
	"github.com/timetracker/backend/internal/infrastructure/persistence/db"
	_ "gorm.io/driver/postgres"
	// GORM IMPORTS MARKER
)

// @title Time Tracker API
// @version 1.0
// @description This is a simple time tracker API.
// @host localhost:8080
// @BasePath /

// @Summary Say hello
// @Description Get a hello message
// @ID hello
// @Produce  plain
// @Success 200 {string} string "Hello from the Time Tracker Backend!"
// @Router / [get]
func helloHandler(c *gin.Context) {
	c.String(http.StatusOK, "Hello from the Time Tracker Backend!")
}

func main() {

	db, _ := db.NewDatasourceContainer(db.DatabaseConfig{
		Host:     "localhost",
		Port:     5432,
		User:     "timetracker",
		Password: "timetracker",
		DBName:   "timetracker",
		SSLMode:  "disable",
	})

	r := gin.Default()

	r.GET("/", helloHandler)
	r.GET("/swagger/*any", ginSwagger.WrapHandler(swaggerFiles.Handler))

	fmt.Println("Server listening on port 8080")
	r.Run(":8080") // Use Gin's Run method
}
